/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fileManager;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jdom.JDOMException;
import org.vast.sensorML.SMLUtils;
import org.vast.xml.XMLReaderException;
import org.vast.xml.XMLWriterException;

import net.opengis.sensorml.v20.PhysicalComponent;
import net.opengis.sensorml.v20.PhysicalSystem;
import net.opengis.sensorml.v20.SimpleProcess;

/**
 * This class implements functionalities to read information from the intelligent sensor's description
 * that has to be available locally at the device.
 * The description files has to be available from path "System/'Use Case Folder Name'/INTELLIGENT_SENSOR.xml".
 * @author A_Fritze
 *
 */
public class Read {

	private SMLUtils smlUtils = new SMLUtils(SMLUtils.V2_0);
	
	protected PhysicalSystem Isens;
	private int numSens, numAlgs;
	private List<PhysicalComponent> listOfSolidSens;
	private List<SimpleProcess> listOfAlgorithms;
	
	/**
	 * This class implements functionalities to read information from the intelligent sensor's description
	 * that has to be available locally at the device.
 	 * The description files has to be available from the path avaialable in the
 	 * local variable path.
 	 * @author A_Fritze
	 */
	public Read(String folderNameSelfDescription){	
		try {
			this.Isens = (PhysicalSystem) smlUtils.readProcess(new FileInputStream(new File("System/"+ folderNameSelfDescription +"/INTELLIGENT_SENSOR.xml")));
		} catch (XMLReaderException | FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			
			System.out.println("!!!!!!!Could not read description file of Intelligent Sensor!!!!!!!");
			System.out.println("!!!!!!!Make sure that the file is located at correct directory (project dir/System/INTELLIGENT_SENSOR.xml) at the device!!!!!!!");
			
			System.exit(0);
		}
		numSens=0;
		numAlgs=0;
		
		if(Isens == null){
			System.out.println("!!!!!!!Could not read description file of Intelligent Sensor!!!!!!!");
			System.out.println("!!!!!!!Make sure that the file is located at correct directory at the device!!!!!!!");
			System.exit(0);
		}
		
		listOfSolidSens = new ArrayList<PhysicalComponent>();
		listOfAlgorithms = new ArrayList<SimpleProcess>();
		
		
		try {
			extractComponents(Isens);
		} catch (IOException | JDOMException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.out.println("a");
		}
	}	
	
	//###############################################################################################
	//###############################################################################################
	public String getNameIsens(){
		String name;	
		name = Isens.getIdentificationList().get(0).getIdentifier2List().get(0).getValue();
		return name;
	}
	//###############################################################################################
	//###############################################################################################
	public String getUIDIsens(){
		String name;	
		name = Isens.getIdentifier().getValue();
		return name;
	}
	//###############################################################################################
	//###############################################################################################
	public int getNumSens() throws JDOMException, IOException{
		return numSens;
	}
	//###############################################################################################
	//###############################################################################################	
	public int getNumAlg() throws Exception{
		return numAlgs;
	}
	
	//###############################################################################################
	//###############################################################################################
	public String getSensorClassifierTerm(int numSensor, String nameTerm){			
		for(int i = 0; i< listOfSolidSens.get(numSensor).getClassificationList().get(0).getClassifierList().size(); i++){
			
			if(listOfSolidSens.get(numSensor).getClassificationList().get(0).getClassifierList().get(i).getLabel().equals(nameTerm))
				return listOfSolidSens.get(numSensor).getClassificationList().get(0).getClassifierList().get(i).getValue();
		}
		return null;
	}
	//###############################################################################################
	//###############################################################################################
	public String getSensorFOIHref(int numSensor){
		String foiS = listOfSolidSens.get(numSensor).getFeaturesOfInterest().getFeatureList().getProperty(0).getHref();		
		return foiS;
	}
	//###############################################################################################
	//###############################################################################################
	public String getSensorFOITitle(int numSensor){
		String foiS = listOfSolidSens.get(numSensor).getFeaturesOfInterest().getFeatureList().getProperty(0).getTitle();		
		return foiS;
	}
	//###############################################################################################
	//###############################################################################################
	public String getSensorUID(int numSensor){
		String uid = listOfSolidSens.get(numSensor).getIdentifier().getValue();
		return uid;		
	}
	//###############################################################################################
	//###############################################################################################
	public String getAlgorithmUID(int numAlg){
		String uid = listOfAlgorithms.get(numAlg).getIdentifier().getValue();
		return uid;
	}
	
	//###############################################################################################
	//###############################################################################################
	public String getAlgorithmCodePath(int numAlg){
		for(int i = 0; i< listOfAlgorithms.get(numAlg).getIdentificationList().get(0).getIdentifier2List().size(); i++){			
			if(listOfAlgorithms.get(numAlg).getIdentificationList().get(0).getIdentifier2List().get(i).getLabel().equals("Code Path"))
				return listOfAlgorithms.get(numAlg).getIdentificationList().get(0).getIdentifier2List().get(i).getValue();
		}
		return null;
	}
	//###############################################################################################
	//###############################################################################################
	public int getSensorOutputDimension(int numSensor){
		int dimSens = listOfSolidSens.get(numSensor).getOutputList().getComponent(0).getComponent("dimensionality").getData().getIntValue();
		return dimSens;		
	}
	//###############################################################################################
	//###############################################################################################
	public int getSensorOutputDataLength(int numSensor){
		int dimSens = listOfSolidSens.get(numSensor).getOutputList().getComponent(0).getComponent("dataLength").getData().getIntValue();
		return dimSens;		
	}
	//###############################################################################################
	//###############################################################################################
	public String getSensorOutputDataType(int numSensor){
		String type = listOfSolidSens.get(numSensor).getOutputList().getComponent(0).getComponent("dataType").getLabel();
		return type;		
	}
	//###############################################################################################
	//###############################################################################################
	public String getSensorInterfaceType(int numSensor){
		String type = listOfSolidSens.get(numSensor).getOutputList().getComponent(0).getComponent("interface").getLabel();
		return type;		
	}
	//###############################################################################################
	//###############################################################################################
	public int getSensorInterfaceChannel(int numSensor){
		int channel = listOfSolidSens.get(numSensor).getOutputList().getComponent(0).getComponent("channel").getData().getIntValue();
		return channel;		
	}
	//###############################################################################################
	//###############################################################################################
	public int getAlgorithmOutputDimension(int numAlg){
		int dimAlg = listOfAlgorithms.get(numAlg).getOutputList().getComponent(0).getComponent("dimensionality").getData().getIntValue();
		
		return dimAlg;		
	}	
	//###############################################################################################
	//###############################################################################################
	public int getAlgorithmInputDimension(int numAlg){
		int dimAlg = listOfAlgorithms.get(numAlg).getInputList().getComponent(0).getComponent("dimensionality").getData().getIntValue();
		return dimAlg;		
	}
	//###############################################################################################
	//###############################################################################################
	public String getAlgorithmOutputPhysQuant(int numAlg){
		String physQ = listOfAlgorithms.get(numAlg).getOutputList().getComponent(0).getComponent("type").getLabel();
		return physQ;
	}
	//###############################################################################################
	//###############################################################################################
	public String[] getAlgorithmInputPhysQuant(int numAlg){		
		ArrayList<String> names = new ArrayList<String>();
		
		//momentan nur eine PhysQuant pro algorithmus
		for(int i = 0; i<listOfAlgorithms.get(numAlg).getInputList().getComponent(0).getComponentCount(); i++){
			
			if(!listOfAlgorithms.get(numAlg).getInputList().getComponent(0).getComponent(i).getName().equals("dimensionality")){
				//System.out.println(listOfAlgorithms.get(numAlg).getInputList().getComponent(0).getComponent(i).getName());
				names.add(listOfAlgorithms.get(numAlg).getInputList().getComponent(0).getComponent(i).getLabel());
			}
		}
		String[] inPhyQunat = new String[names.size()];
		
		for(int i=0; i<names.size();i++){
			inPhyQunat[i] = names.get(i);
		}
		
		//System.out.println(listOfAlgorithms.get(numAlg).getInputList().getComponent(0).getComponentCount());
		return inPhyQunat;
	}
	
	//###############################################################################################
	/**
	 * Read Components of PhysicalSystem. Components are either elementary sensors or algorithms.
	 * @param sys - the intelligent sensor 
	 * @throws XMLWriterException
	 * @throws IOException
	 * @throws JDOMException
	 */
	private void extractComponents(PhysicalSystem sys) throws XMLWriterException, IOException, JDOMException{
				
		//Number of intelligent sensors
		int numComponents;
		
		numComponents = sys.getComponentList().size();
		//System.out.println(numComponents);
		
		for(int i=0; i<numComponents;i++){			
			if(sys.getComponentList().getProperty(i).getArcRole().equals("sensor")){
				numSens++;
				listOfSolidSens.add((PhysicalComponent)smlUtils.readProcess(new FileInputStream(new File(sys.getComponentList().getProperty(i).getHref()))));
			}
			if(sys.getComponentList().getProperty(i).getArcRole().equals("algorithm")){
				numAlgs++;
				//System.out.println("Alg" + numAlgs + " ");
				listOfAlgorithms.add((SimpleProcess)smlUtils.readProcess(new FileInputStream(new File(sys.getComponentList().getProperty(i).getHref()))));
			}
		}
	}	
}
