/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;


import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

public class FeatureConfigThread extends Thread {

	
	//static ServerSocket variable
    private  ServerSocket server;
    //socket server port on which it will listen
    private int port;
    
    private boolean stop = false;
    
    private int featurePort = 9901;
    
    private FeatureStore featureStore;
    
    ObjectInputStream ois;
    ObjectOutputStream oos;
    Socket socket;
	
	
	public FeatureConfigThread(int port, FeatureStore featureStore){
		this.port = port;
		this.featureStore = featureStore;
	}	
	
	public void run(){
		//create the socket server object
        try {
			server = new ServerSocket(port);
			System.out.println("Feature-Config port: " + server.getLocalPort());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
		while(!stop){
			try{
				sleep(200);
			}
			catch(InterruptedException ex){
				Thread.currentThread().interrupt();
			}
			
			listenToClient();
		}
	}
	
	/**
	 * Waits for a client message and acts depending on the message.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public void listenToClient() {
		try{
			//creating socket and waiting for client connection
	        socket = server.accept();
	        //System.out.println("Socket Port: " + socket.getLocalPort());
	        //read from socket to ObjectInputStream object
	        ois = new ObjectInputStream(socket.getInputStream());
	        //convert ObjectInputStream object to String
	        String message = (String) ois.readObject();
	        System.out.println("Port "+ port +": " + message);
	        
	        String[] splitMessage = message.split("_");
	        
	        //create ObjectOutputStream object
	        oos = new ObjectOutputStream(socket.getOutputStream());
	        switch (splitMessage[0]) {
			case "feature":
				//System.out.println("Initialise feature " + splitMessage[1]);
		        //return port of input data for the new feature
		        oos.writeObject(featurePort);
	        	        
	        
		        Algorithm alg = featureStore.getAlgorithm(splitMessage[1]);
		        
		        //Add feature to feature store and set-up all data-interfaces (ports)
		        Feature feature = new Feature(splitMessage[2], alg, featurePort);
				featureStore.addFeature(feature);
		        
		        //One port for input and one port for output
		        featurePort = featurePort + 2;
		        
		        //close resources
		        ois.close();
		        oos.close();
		        break;
			case "clear":
				//System.out.println("Clear features");
				featurePort = 9901;
				
				for(int i = 0; i<featureStore.getFeatureList().size();i++){
					featureStore.getFeatureList().get(i).closeThreads();
				}
				featureStore.clear();
		        //close resources
		        ois.close();
		        oos.close();
		        break;
			default :
				System.out.println("FeatureConfig Port: message can not be interpreted");
				oos.writeObject("Received message can not be interpreted");
		        //close resources
		        ois.close();
		        oos.close();
				break;
			}
	        socket.close();
		}catch(Exception e){
			e.getMessage();
		}
	}
	
	/**
	 * Closes this thread.
	 */
	public void closeThread(){
		stop=true;
		try {
			server.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}

