/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;

public class FeatureInThread extends Thread{
	//static ServerSocket variable
    private ServerSocket server;
    //socket server port on which it will listen
    private int port;
    
    private boolean stop = false;
    
    private Feature feature;
    //Latest value
    private Object value;
    
    ObjectInputStream ois;
    ObjectOutputStream oos;
    Socket socket;
	
	public FeatureInThread(int port, Feature feature){
		this.port = port;
		this.feature = feature;
	}	
	
	public void run(){
		//create the socket server object
	    try {
			server = new ServerSocket(port);
			//SocketAddress endpoint = new InetSocketAddress("localhost", port);
			//server.bind(endpoint);
			//initSocket=true;
			//server.setSoTimeout(50);
			//System.out.println("Feature Input port: " + server.getLocalPort());
		} catch (IOException e) {
			e.printStackTrace();
		}
		
        
		while(!stop){
			try{
				sleep(200);
			}
			catch(InterruptedException ex){
				Thread.currentThread().interrupt();
			}
			
			listenToClient();
		}
	}
	
	/**
	 * Waits for a client message and acts depending on the message.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public void listenToClient(){
		//creating socket and waiting for client connection
		if(!server.isClosed()){
			try{
				socket = server.accept();
		        //read from socket to ObjectInputStream object
		        ois = new ObjectInputStream(socket.getInputStream());
		        //create ObjectOutputStream object
		        oos = new ObjectOutputStream(socket.getOutputStream());        
		        oos.writeObject("ok");
	        	oos.flush();
		        oos.reset();
		        while(!stop){
		        	
		        	Object message = ois.readObject();
		        	
		        	Double inMessage = ByteBuffer.wrap((byte[]) message).getDouble();
		        	//System.out.println("Message on port " + port +": "+inMessage);
		        	if(inMessage.getClass().equals(Double.class)){
		        		
			        	value = (double) inMessage;
				        //System.out.println("\nMessage Received on Port "+ port +": " + value);
				        
				        feature.computeFeatureValue(value);
				        //System.out.println("Received value on port " + port + ": " + inMessage + "\nFeature Value: ");
				        
				        oos.writeObject("ok");
				        //System.out.println("Send ok");
			        	oos.flush();
				        oos.reset();				        
			        }
			        else{
			        	oos.writeObject("x");
			        	oos.flush();
				        oos.reset();
			        	//System.out.println("\nMessage Received on Port "+ port +": Message can not be interpreted");
			        }				    
			        //socket.close();
		        }
		        if(stop){
		        	oos.close();
		        	ois.close();
		        	server.close();
					System.out.println("Closed Feature Input Socket");
				}
			}catch(Exception e){
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Closes this thread.
	 */
	public void closeThread(){
		stop=true;
		try {
			server.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
