/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.util.Arrays;

public class FeatureOutThread extends Thread{
	//static ServerSocket variable
    private ServerSocket server;
    //socket server port on which it will listen
    private int port;
    
    private boolean stop = false;
    
    private Feature feature;
    //Latest value
    private Object value;
	
	public FeatureOutThread(int port, Feature feature){
		this.port = port;
		this.feature = feature;
	}
	
	ObjectInputStream ois;
    ObjectOutputStream oos;
    Socket socket;
	
	public void run(){
		//create the socket server object
        try {
			server = new ServerSocket(port);
			//server.setSoTimeout(2000);
			//System.out.println("Send from : " + feature.UID + server.getLocalPort());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
		while(!stop){
			try{
				sleep(200);
			}
			catch(InterruptedException ex){
				Thread.currentThread().interrupt();
			}
			
			
			listenToClient();
		}
	}
	
	/**
	 * Waits for a client message and acts depending on the message.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public void listenToClient(){
		//creating socket and waiting for client connection
		if(!server.isClosed()){
			try{
				socket = server.accept();
		        //System.out.println("Socket Port: " + socket.getLocalPort());
		        //read from socket to ObjectInputStream object
		        ois = new ObjectInputStream(socket.getInputStream());
		        //create ObjectOutputStream object
		        oos = new ObjectOutputStream(socket.getOutputStream());
		        
		        while(!stop){
			        //convert ObjectInputStream object to String
			        String message = (String) ois.readObject();
			        //System.out.println("Message Received on Feature "+ port +": " + message);
			        
			        
			        switch (message) {
					case "data":
				        //write object to Socket
				        value = feature.memValue;
				        //System.out.println("Output  " + feature.UID + "   port"+ port +": " + value);
				        if(value!=null){
				        	byte[] outMessage = ByteBuffer.allocate(8).putDouble((double) value).array();
				        	oos.writeObject(outMessage);
				        	oos.flush();
					        oos.reset();
				        }else{
				        	byte outMessage[] = new byte[8];
				        	Arrays.fill(outMessage, (byte) 0xff);
				        	oos.writeObject(outMessage);
				        	oos.flush();
					        oos.reset();
				        }
				        break;
					default :
						oos.writeObject("Received message can not be interpreted");
						oos.flush();
				        oos.reset();
						break;
					}
			        //socket.close();
		        }
		        if(stop){
		        	oos.close();
		        	ois.close();
					server.close();
				}
			}catch(Exception e){
				e.printStackTrace();
			}	        
		}
	}
	
	/**
	 * Closes this thread.
	 */
	public void closeThread(){
		stop=true;
		try {
			server.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
