/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;

import java.util.List;

public interface FeatureServerInterface {
	
	/**
	 * Compute the value of this feature. This method also updates the membership value.
	 * @param data
	 * @return
	 */
	public Object computeFeatureValue(Object data);
	
	/**
	 * Train the membership function of this feature
	 * @param data list of training data
	 * @param parameters list of parameters
	 */
	public void trainMembershipFunction(List<Double> data);
	
	/**
	 * Returns the current membership value of the current feature value
	 * @return current membership value
	 */
	public double computeMembershipValue(double input);
	Object computeValue(Object input);
	
	public boolean isTrained();
	
	public void loadAlgorithmClass();
	
	//Getter
	public String getUID();
	public Algorithm getAlgorithm();
	
	//Add
	public void addSensor(Sensor sensor);

	

}
