/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.nio.ByteBuffer;
import com.pi4j.io.i2c.I2CBus;
import gpioreader.Pi4JUART;
import gpioreader.Pi4JUSBSer;
import gpioreader.Pi4jGPIO;

public class SensorThread extends Thread {

	
	//static ServerSocket variable
    private  ServerSocket server;
    //socket server port on which it will listen
    private int port;
    
    private boolean stop = false;
	
	//GPIO object
	Pi4jGPIO gpio;
	int channel;
	
	//Instance of the serial communications class
    Pi4JUART serial;
    
    Pi4JUSBSer usbSerial;
	
    //Instance of the I2C communication
    I2CBus i2c;
    //I2CDevice device;
        
    //Constants 
	final int TYPEGPIO = 1;
	final int TYPEUART = 2;
	final int TYPEUSBSERIAL = 3;
	final int TYPEI2C = 4;
	private int type;
	
	private Object value = null;
	
	/**
	 * This thread implements the interface to elementary sensors. Available types are
	 * GPIO (type=1)
	 * UART (type=2)
	 * @param type
	 * @param GPIOChannel
	 * @param port
	 */
	public SensorThread(int type, int channel, int port){
		this.port = port;
		this.type = type;
		// init GPIO
		if(type==TYPEGPIO){
			this.channel = channel;
			try {
				this.gpio = new Pi4jGPIO(channel);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		// init UART
		else if(type==TYPEUART){
			serial = new Pi4JUART();
		}
		// init UART
		else if(type==TYPEUSBSERIAL){
			this.channel = channel;
		}
		else if(type==TYPEI2C){
			/*
			try {
				i2c = I2CFactory.getInstance(I2CBus.BUS_1);
				// create an I2C device for an individual device on the bus that you want to communicate with
		        // in this example we will use the default address for the TSL2561 chip which is 0x39.
		        //device = i2c.getDevice(TSL2561_ADDR);
		        
		        //Power up sensor
		        //device.write(TSL2561_REG_CONTROL, TSL2561_POWER_UP);
		        
		        // wait while the chip collects data
		        Thread.sleep(500);
		        
			} catch (IOException | InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			*/			
		}
		else{
			System.out.println("No type for type=" + type + " exists.");
		}		
	}	
	
	public void run(){
		//create the socket server object
        try {
			server = new ServerSocket(port);
			//System.out.println("Sensor port: " + server.getLocalPort());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
		while(!SensorThread.currentThread().isInterrupted()){
			try{
				sleep(200);
			}
			catch(InterruptedException ex){
				Thread.currentThread().interrupt();
			}
			
			listenToClient();
		}
	}
	
	public void setUSBSerial(Pi4JUSBSer usbSerial2){
		this.usbSerial = usbSerial2;		
	}
	
	/**
	 * Waits for a client message and acts depending on the message.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public void listenToClient(){
		try{
			//creating socket and waiting for client connection
	        Socket socket = server.accept();
	        //System.out.println("Socket Port: " + socket.getLocalPort());
	        //read from socket to ObjectInputStream object
	        ObjectInputStream ois = new ObjectInputStream(socket.getInputStream());
	        //create ObjectOutputStream object
	        ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
	        
	        while(!stop){
		        //convert ObjectInputStream object to String
		        String message = (String) ois.readObject();
		        //System.out.println("Message Received on Sensor "+ port +": " + message);
		        
		        switch (message) {
				case "data":
			        //write object to Socket
			        value = getCurrentSensValue();
			        
			        value = Math.abs((double)value);
			        //value = Math.random();
			        //System.out.println("Send on Port " + port + ":  " + value);
			        byte[] outMessage = ByteBuffer.allocate(8).putDouble((double) value).array();
			        
			        oos.writeObject(outMessage);
			        oos.flush();
			        oos.reset();
			        //close resources
			        //ois.close();
			        //oos.close();
			        break;
				default :
					oos.writeObject("Received message can not be interpreted");
			        //close resources
			        //ois.close();
			        //oos.close();
					break;
		        }
	        //socket.close();
	        }
	        if(stop){
	        	ois.close();
	        	oos.close();
	        	socket.close();
				server.close();
				
				if(type==TYPEI2C){
					//device.write(TSL2561_REG_CONTROL, TSL2561_POWER_DOWN);
				}
				
				this.interrupt();
			}
		}catch(IOException | ClassNotFoundException e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Returns the actual sensor measurement. The computation of the measurement has to be implemented in this method.
	 * @return Current sensor observation
	 */
	public Object getCurrentSensValue(){		
		if(type==TYPEGPIO){
			value = gpio.getValue();
			return value;
		}
		else if(type==TYPEUART){
			return serial.getValue();
		}
		else if(type==TYPEUSBSERIAL){
			usbSerial.requestData();
			do{
				try {
					Thread.sleep(20);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}while(!usbSerial.receivedUSBSerial);
			usbSerial.receivedUSBSerial=false;
			return usbSerial.getValue(channel);
		}
		else if(type==TYPEI2C){			
		}
		return null;				
	}
	
	/**
	 * Closes this thread.
	 */
	public void closeThread(){
		stop=true;
	}
}
