/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package gpioreader;

import java.io.IOException;
import java.nio.ByteBuffer;

import com.pi4j.io.gpio.exception.UnsupportedBoardType;
import com.pi4j.io.serial.Baud;
import com.pi4j.io.serial.DataBits;
import com.pi4j.io.serial.FlowControl;
import com.pi4j.io.serial.Parity;
import com.pi4j.io.serial.Serial;
import com.pi4j.io.serial.SerialConfig;
import com.pi4j.io.serial.SerialDataEvent;
import com.pi4j.io.serial.SerialDataEventListener;
import com.pi4j.io.serial.SerialFactory;
import com.pi4j.io.serial.SerialPort;
import com.pi4j.io.serial.StopBits;

public class Pi4JUART {
	
	Object value;	

	public Pi4JUART(){
		
		value = null;
		// create an instance of the serial communications class
	    final Serial serial = SerialFactory.createInstance();

	    // create and register the serial data listener
	    serial.addListener(new SerialDataEventListener() {
	        @Override
	        public void dataReceived(SerialDataEvent event) {
	        	
	            // NOTE! - It is extremely important to read the data received from the
	            // serial port.  If it does not get read from the receive buffer, the
	            // buffer will continue to grow and consume memory.

	            // print out the data received to the console
	            try {
	            	value = ByteBuffer.wrap(event.getBytes()).getDouble();
                    //System.out.println(event.getAsciiString());
	            } catch (IOException e) {
	                e.printStackTrace();
	            }
	        }
	    });
	    
	    SerialConfig config = new SerialConfig();
        //Pi4jGPIODigi gpio1 = new Pi4jGPIODigi();
        
        // set default serial settings (device, baud rate, flow control, etc)
        //
        // by default, use the DEFAULT com port on the Raspberry Pi (exposed on GPIO header)
        // NOTE: this utility method will determine the default serial port for the
        //       detected platform and board/model.  For all Raspberry Pi models
        //       except the 3B, it will return "/dev/ttyAMA0".  For Raspberry Pi
        //       model 3B may return "/dev/ttyS0" or "/dev/ttyAMA0" depending on
        //       environment configuration.
        try {
			config.device(SerialPort.getDefaultPort())
			      .baud(Baud._115200)
			      .dataBits(DataBits._8)
			      .parity(Parity.NONE)
			      .stopBits(StopBits._1)
			      .flowControl(FlowControl.NONE);
			
			// open the default serial device/port with the configuration settings
	        serial.open(config);
	        
		} catch (UnsupportedBoardType | IOException | InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public Object getValue() {
	        return value;
	}    
}
