/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.util.concurrent.Semaphore;

public class AttributeInThread extends Thread{
	//static ServerSocket variable
    private ServerSocket server;
    //socket server port on which it will listen
    private int port;
    //the id is required to separate the inputs of the attribute from each other
    private int id;
    
    //Variable to close the socket and interrupt (stop) this thread
    private boolean stop = false;
    
    //Semaphore to manage multiple access to fusion method
    private Semaphore semaphore;
        
    Attribute attribute;
    
    ObjectInputStream ois;
    ObjectOutputStream oos;
    Socket socket;
	
	public AttributeInThread(int port, Attribute attribute, int id, Semaphore semaphore){
		this.port = port;
		this.attribute = attribute;
		this.id = id;
		this.semaphore = semaphore;
	}
	
	public void run(){
		//create the socket server object
        try {
        	
			server = new ServerSocket(port);
			//server.setSoTimeout(3000);
			
		} catch (IOException e) {
			System.out.println("Input port: " + port);
			e.printStackTrace();
		}
        
		while(!stop){
			try{
				sleep(200);
			}
			catch(InterruptedException ex){
				Thread.currentThread().interrupt();
			}
			
			listenToClient();
		}
	}
	
	/**
	 * Waits for a client message and acts depending on the message.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 * @throws InterruptedException 
	 */
	public void listenToClient(){
		//creating socket and waiting for client connection
		if(!server.isClosed()){
			try{
				socket = server.accept();
		        //read from socket to ObjectInputStream object
		        ois = new ObjectInputStream(socket.getInputStream());
		        //create ObjectOutputStream object
		        oos = new ObjectOutputStream(socket.getOutputStream());
		        oos.writeObject("ok");
	        	oos.flush();
		        oos.reset();		        
		        while(!stop){
		        	Object message = ois.readObject();
		        	Double inMessage = ByteBuffer.wrap((byte[]) message).getDouble();
		        	//System.out.println(attribute.getUID() +" at port " + port + ": " + inMessage);		        	
		        	if(inMessage.getClass().equals(Double.class)){
		        		//Update attribute and fuse information
		        		semaphore.acquire();
			        	attribute.setBBAandFuse(id, inMessage);
			        	semaphore.release();
			        	oos.writeObject("ok");
			        	oos.flush();
				        oos.reset();
			        }
			        else{
			        	System.out.println("\nMessage Received on Port "+ port +": Message can not be interpreted");
			        	oos.writeObject("x");
			        	oos.flush();
				        oos.reset();
			        }		        	
			        //socket.close();
		        }
		        if(stop){
		        	oos.close();
		        	ois.close();
					server.close();
				}
			}catch(Exception e){
				e.printStackTrace();
			}
			
			
		}
	}
	
	/**
	 * Closes this thread.
	 */
	public void closeThread(){
		stop=true;		
		try {
			//oos.close();
			//ois.close();
			server.close();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.getMessage();
		}
	}
}
