/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager;


public class Sensor implements SensorClientInterface {
	
	// check if data interface is initialized
	boolean isInitDataInterface = false;
	
	private String UID;
	private String physPheno;
	private int dimensionality;
	private String foi;
	private String dataType;
	private int dataLength;
	private String sourceIP;
	private int port;
	
	/**
	 * The Sensor-Class represents an elementary sensor of an intelligent sensor. Semantic data is required to generate an sensor object.
	 * For data exchange an interface can be created using the initDataInterface-Class.
	 * This is not required if only semantics are required.
	 * 
	 * @param UID Distinct name of the elementary sensor
	 * @param suitablePhysPheno Physical phenomenon the sensor states about
	 * @param signalDimensionality Dimensionality of the output signal
	 * @param dataType Data type of the returned measurement
	 * @param sourceIP Network address (same as for the intelligent sensor)
	 * @param observedEntity The entity (Module, area, product, etc.) the sensor observes
	 */
	public Sensor(String UID, String suitablePhysPheno, int signalDimensionality, String dataType, int dataLength, String sourceIP, int port, String observedEntity){

		//Semantics
		this.UID = UID;
		this.physPheno = suitablePhysPheno;
		this.foi = observedEntity;
		this.dimensionality = signalDimensionality;
		this.dataType = dataType;
		this.dataLength = dataLength;
		this.sourceIP = sourceIP;
		this.port = port;
	}
	
	@Override
	public String getUID() {
		return UID;
	}

	@Override
	public String getfoi() {		
		return foi;
	}

	@Override
	public String getPhysPheno() {
		return physPheno;
	}

	@Override
	public int getOutputDimensonality() {
		return dimensionality;
	}	

	@Override
	public String getDataType() {
		return dataType;
	}
	
	@Override
	public int getDataLength() {
		return dataLength;
	}

	@Override
	public String getIP() {
		return sourceIP;
	}

	@Override
	public int getPort() {
		return port;
	}
}
