/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package fusion.manager.aggregation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class BalTLCS {
	
	private List<Double> BBAs = new ArrayList<Double>();		// basic belief assignments of every sensor
	private int n = 0;											// no. of input features
	
	double m_nc;												// non-conflicting mass
	double k_cm;												// conflicting factor
	double m_c;													// conflicting mass
	double m_normal;											// Overall score value for the attribute
	double importance;											// importance measure for the attribute
	
	// Result-List ++++++++ values[0]=m_normal, values[1]=k_cm, values[2]=importance, values[3]=m_nc, values[4]=m_c
	List<Double> values = Arrays.asList((double) 0, (double) 0, (double) 0, (double) 0, (double) 0);				
	
	double sum = 0;
	
	/**
	 * Fuse information using BalTLCS. The implementation is only for having two classes.
	 * @param BBAs Basic Belief Assignments that should be fused
	 * @return returns an float list values[] with values[0]=m_normal, values[1]=k_cm, values[2]=importance, values[3]=m_nc, values[4]=m_c
	 */
	public List<Double> fuse(List<Double> BBAs){
		this.BBAs = BBAs;
		this.n = BBAs.size();
		
		m_nc = compute_mnc();									// non-conflicting mass
		k_cm = compute_kcm(m_nc);								// conflicting factor
		importance = 1 - k_cm;									// importance measure
		m_c  = compute_mc(k_cm);								// conflicting mass
		
		m_normal = m_c + m_nc;
		
		values.set(0, m_normal);
		values.set(1, k_cm);
		values.set(2, importance);
		values.set(3, m_nc);		
		values.set(4, m_c);
		
		return values;
	}
	
	/**
	 *  Compute the non-conflicting part
	 * @return
	 */
	private double compute_mnc(){
		sum = 0;
		
		for(int i = 0; i < (n - 1); i++ ){
			for(int j = (i+1); j < n; j++ ){
				sum = sum + (BBAs.get(i) * BBAs.get(j));			
			}			
		}
		
		return (2 * sum) / (n * (n - 1));
	}
	
	/**
	 *  Compute the conflicting coefficient
	 * @param mnc: Non-conflicting masses
	 * @return
	 */
	private double compute_kcm(double mnc){
		sum = 0;
		
		for(int i = 0; i < n; i++ ){		
			sum = sum + BBAs.get(i);
		}
		
		return 2 * ( (sum / n) - mnc);	
	}
	
	/**
	 *  Compute the conflicting part
	 * @param kcm: conflicting factor
	 * @return
	 */
	private double compute_mc(double kcm){
		sum = 0;
		
		for(int i = 0; i < n; i++ ){		
			sum = sum + BBAs.get(i);
		}
		
		return  kcm * (sum / n);
	}	
}
