/*
 * Copyright 2017 inIT - Institute Industrial IT

	This file is part of "Automated Fusion System Design and Adaptation Implementation.

    "Automated Fusion System Design and Adaptation Implementation is free software: 
	you can redistribute it and/or modify it under the terms of the GNU General Public 
	License as published by the Free Software Foundation, either version 3 of the 
	License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT ANY 
	WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
	PARTICULAR PURPOSE. See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along with this 
	program.  If not, see <http://www.gnu.org/licenses/>.
	
 * A copy of the GNU General Public License version 3 is provided in the file LICENSE.gpl3.0.txt .
 */
package opcua.client;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

public class main {
	
	// Address of the discovery server. Initially set to local host.
	protected static String discoveryServerUrl = "opc.tcp://localhost:4840";
	private static boolean isSetPathToDescription = false;
	private static String folderNameDescriptionFiles = "";

	public static void main(String[] args) throws Exception {
		// Initialize log4j logging
		PropertyConfigurator.configure("log.properties");
		
		//Switch off all logging
		Logger.getRootLogger().setLevel(Level.OFF);
		Logger.getLogger("org.opcfoundation.ua").setLevel(Level.OFF);
		Logger.getLogger("com.prosysopc.ua").setLevel(Level.OFF);
		try {
			if (!parseCmdLineArgs(args)) {
				usage();
				return;
			}
		} catch (IllegalArgumentException e) {
			System.out.println("Invalid cmd line argument: " + e.getMessage());
			usage();
			return;
		}
		
		if(!isSetPathToDescription){
			System.out.println("Please set path to system describtion files."
					+ " Start with 'sudo java -jar clientRPI.jar -?' for more information.");
			System.exit(0);
		}
		
		ClientMain client = new ClientMain(folderNameDescriptionFiles);
		
		client.start(discoveryServerUrl);
		System.exit(0);
	}
	/**
	 * Read application parameters
	 */
	protected static boolean parseCmdLineArgs(String[] args)
			throws IllegalArgumentException {
		if(args.length == 0){
			throw new IllegalArgumentException();
		}
		else if(!args[0].startsWith("-")){
			throw new IllegalArgumentException();
		}
		int i = 0;
		while ((args.length > i) && ((args[i].startsWith("-") || args[i].startsWith("/")))) {
			if (args[i].equals("-d"))
				discoveryServerUrl =  "opc.tcp://" + args[++i] + ":4840";
			else if (args[i].equals("-?") || args[i].equals("-h") || args[i].equals("-help"))
				return false;
			else if (args[i].equals("-desc")){
				folderNameDescriptionFiles = args[++i];
				isSetPathToDescription = true;
			}
			else
				throw new IllegalArgumentException(args[i]);
			i++;
		}
		return true;
	}
	/**
	 * Displays how to use the application.
	 */
	protected static void usage(){
		System.out.println("Usage: " + "sudo java -jar clientRPI.jar" );
		System.out.println("   -d 'url'     Define the DiscoveryServerUrl to register the application to");
		System.out.println("   -desc 'folder name'         Set folder name of system descriptions for the use case scenario. The files have to be availabe from the 'System' folder that has to be located in the same directory as the programm file.");
		System.out.println("   -?; -h; -help         Show this help text");
		System.out.println("");
	}
}
